<?php
/**
 * Gmaper for Elementor
 * Google maps for Elementor editor
 * Exclusively on https://1.envato.market/gmaper-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.4
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\GmaperElementor;

use Merkulove\GmaperElementor\Unity\Plugin;
use Merkulove\GmaperElementor\Unity\Settings;
use Merkulove\GmaperElementor\Unity\TabGeneral;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Settings class used to modify default plugin settings.
 *
 * @since 1.0.0
 *
 **/
final class Config {

	/**
	 * The one true Settings.
	 *
     * @since 1.0.0
     * @access private
	 * @var Config
	 **/
	private static $instance;

    /**
     * Prepare plugin settings by modifying the default one.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function prepare_settings() {

        /** Get default plugin settings. */
        $tabs = Plugin::get_tabs();

        $offset = 0;
        $tabs = array_slice( $tabs, 0, $offset, true ) +
            ['google_maps_api' => [
                'enabled'       => true,
                'class'         => TabGeneral::class, // Handler
                'label'         => esc_html__( 'Google Maps API Key', 'gmaper-elementor' ),
                'title'         => esc_html__( 'Google Maps API Key', 'gmaper-elementor' ),
                'show_title'    => true,
                'icon'          => 'map', // Icon for tab
                'fields'        => []
            ] ] +
            array_slice( $tabs, $offset, NULL, true );
        $backend_options = is_array( get_option( 'mdp_stellar_design_settings' ) );

        $tabs['google_maps_api']['fields']['google_maps_api_key'] = [
            'type'              => 'text',
            'label'             => esc_html__( 'Api key', 'gmaper-elementor' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Api key', 'gmaper-elementor' ),
            'description'       => wp_kses_post( 'Set your api key for Google Maps. Visit <a href="https://docs.merkulov.design/how-to-get-google-map-api-key/" target="_blank">Online Documentation</a> for more details.' ),
            'show_description'  => true,
            'default'           => '',
            'attr'              => [
                'class'     => 'mdp-test-class',
                'maxlength' => '4500'
            ]
        ];

        /** Special config for Elementor plugins. */
        unset( $tabs['general'] );

        /** Set updated tabs. */
        Plugin::set_tabs( $tabs );

        /** Refresh settings. */
        Settings::get_instance()->get_options();

    }

	/**
	 * Main Settings Instance.
	 * Insures that only one instance of Settings exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Config
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
