<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Gmaper for Elementor
 * Google maps for Elementor editor
 * Exclusively on https://1.envato.market/gmaper-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.4
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\GmaperElementor;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Group_Control_Css_Filter;
use Exception;
use Elementor\Widget_Base;
use Elementor\Repeater;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Typography;
use Elementor\Core\Schemes\Typography;
use Elementor\Core\Schemes\Color;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Background;
use Merkulove\GmaperElementor\Unity\Plugin as UnityPlugin;
use Merkulove\GmaperElementor\Unity\ElementorControls;
use Merkulove\GmaperElementor\Unity\Settings;

/** @noinspection PhpUnused */
/**
 * Gmaper - Custom Elementor Widget.
 **/
class gmaper_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {
        $options = Settings::get_instance()->options;

        parent::__construct( $data, $args );

        wp_register_style(
        'mdp-gmaper-elementor-admin',
           UnityPlugin::get_url() . 'src/Merkulove/Unity/assets/css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
               [],
               UnityPlugin::get_version()
        );
        wp_register_style(
            'mdp-gmaper',
               UnityPlugin::get_url() . 'css/gmaper-elementor' . UnityPlugin::get_suffix() . '.css',
                   [],
                   UnityPlugin::get_version()
        );
        wp_register_script(
            'google-maps-api',
               'https://maps.googleapis.com/maps/api/js?key='.$options[ 'google_maps_api_key' ]
        );
        wp_register_script(
             'mdp-gmaper-elementor',
                UnityPlugin::get_url() . 'js/gmaper-elementor' . UnityPlugin::get_suffix() . '.js',
                    [ 'jquery' ],
                    UnityPlugin::get_version(), true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-gmaper-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Gmaper', 'gmaper-elementor' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-gmaper-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'general' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Gmaper' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-gmaper', 'mdp-gmaper-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'google-maps-api', 'mdp-gmaper-elementor' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Section Center Location Settings. */
        $this->section_center_location();

        /** Content -> Section Map Settings. */
        $this->section_map_settings();

        /** Content -> Section Legend Settings. */
        $this->section_legend_settings();

        /** Content -> Section Marker Settings. */
        $this->section_marker_settings();

        /** Content -> Section Map Styles Settings. */
        $this->section_map_styles_settings();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Map. */
        $this->section_style_map();

        /** Style -> Section Style Overlay. */
        $this->section_style_overlay();

        /** Style -> Section Style Marker Title. */
        $this->section_style_marker_title();

        /** Style -> Section Style Marker Description. */
        $this->section_style_marker_description();

        /** Style -> Section Style Legend. */
        $this->section_style_legend();

        /** Style -> Section Style Legend Title. */
        $this->section_style_legend_title();

        /** Content -> Section Style Map Items. */
        $this->section_style_legend_items();

    }

    /**
     * Generates switcher control.
     *
     * @param $switcher_id
     * @param $label
     * @param $default
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_switcher( $switcher_id, $label, $default ) {
        $this->add_control(
            $switcher_id,
            [
                'label' => esc_html__( $label, 'gmaper-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'gmaper-elementor' ),
                'label_off' => esc_html__( 'No', 'gmaper-elementor' ),
                'return_value' => 'yes',
                'default' => $default,
            ]
        );
    }


    /**
     * Add widget controls: Content -> Section Center Location.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_center_location() {

        $this->start_controls_section( 'section_center_location', [
            'label' => esc_html__( 'Center location', 'gmaper-elementor' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'center_latitude',
            [
                'label' => esc_html__( 'Latitude', 'gmaper-elementor' ),
                'label_block' => 'true',
                'type' => Controls_Manager::TEXT,
                'dynamic' => [
                    'active' => true,
                ],
                'condition' => [
                        'center_user_location!' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'center_longitude',
            [
                'label' => esc_html__( 'Longitude', 'gmaper-elementor' ),
                'label_block' => 'true',
                'type' => Controls_Manager::TEXT,
                'dynamic' => [
                    'active' => true,
                ],
                'condition' => [
                        'center_user_location!' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'center_user_location',
            [
                'label' => esc_html__( 'User location', 'gmaper-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'gmaper-elementor' ),
                'label_off' => esc_html__( 'No', 'gmaper-elementor' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Content -> Section Map Settings.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_map_settings() {

        $this->start_controls_section( 'section_map_settings', [
            'label' => esc_html__( 'Map settings', 'gmaper-elementor' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'map_type',
            [
                'label' => esc_html__( 'Map type', 'gmaper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'roadmap',
                'options' => [
                    'roadmap'  => esc_html__( 'Road map', 'gmaper-elementor' ),
                    'satellite' => esc_html__( 'Satellite', 'gmaper-elementor' ),
                    'terrain' => esc_html__( 'Terrain', 'gmaper-elementor' ),
                    'hybrid' => esc_html__( 'Hybrid', 'gmaper-elementor' ),
                ],
            ]
        );


        $this->add_control(
            'map_zoom',
            [
                'label' => esc_html__( 'Zoom', 'gmaper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 21,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 8,
                ],
            ]
        );

        $this->generate_switcher( 'map_overlay', 'Overlay', 'no' );

        $this->generate_switcher( 'scroll_wheel_zoom', 'Scroll wheel zoom', 'no' );

        $this->generate_switcher( 'info_always_open', 'Info container always opened', 'no' );

        $this->generate_switcher( 'info_open_hover', 'Info container open on hover', 'no' );

        $this->add_control(
            'info_container_close',
            [
                'label' => esc_html__( 'Info container close', 'gmaper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'default',
                'options' => [
                    'default' => esc_html__( 'Default', 'gmaper-elementor' ),
                    'click-on-marker'  => esc_html__( 'Click on another marker', 'gmaper-elementor' ),
                    'click-on-map' => esc_html__( 'Click on map', 'gmaper-elementor' )
                ]
            ]
        );

        $this->generate_switcher( 'zoom_controls', 'Zoom controls', 'no' );

        $this->generate_switcher( 'fullscreen_control', 'Full screen control', 'no' );

        $this->generate_switcher( 'disable_map_drag', 'Disable map drag', 'no' );

        $this->generate_switcher( 'show_legend', 'Show legend', 'no' );

        $this->generate_switcher( 'map_type_control', 'Map type control', 'no' );

        $this->generate_switcher( 'street_view_control', 'Street view control', 'no' );

        $this->generate_switcher( 'hide_logo', 'Hide logo', 'no' );

        $this->generate_switcher( 'hide_copyright', 'Hide copyright', 'no' );

        $this->generate_switcher( 'hide_terms_of_use', 'Hide terms of use', 'no' );

        $this->generate_switcher( 'hide_report_error', 'Hide report a map error', 'no' );

        $this->generate_switcher( 'hide_keyboard_shortcuts', 'Hide keyboard shortcuts', 'no' );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Section Legend Settings.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_legend_settings()
    {

        $this->start_controls_section('section_legend_settings', [
            'label' => esc_html__('Legend', 'gmaper-elementor'),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                    'show_legend' => 'yes'
            ]
        ]);

        $this->add_control(
            'legend_title',
            [
                'label' => esc_html__( 'Title', 'gmaper-elementor' ),
                'type' => Controls_Manager::TEXT,
                'dynamic' => [
                    'active' => true,
                ],
                'default' => esc_html__( 'Legend', 'gmaper-elementor' ),
                'placeholder' => esc_html__( 'Type your title here', 'gmaper-elementor' ),
            ]
        );

        $repeater = new Repeater();


        $repeater->add_control(
            'legend_icon',
            [
                'label' => esc_html__( 'Choose Image', 'gmaper-elementor' ),
                'type' => Controls_Manager::MEDIA,
            ]
        );

        $repeater->add_control(
            'legend_icon_size',
            [
                'label' => esc_html__( 'Icon size', 'gmaper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 50,
                ],
                'selectors' => [
                        '{{WRAPPER}} .mdp-gmaper-elementor-google-map-legend--item-image' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};'
                ]
            ]
        );

        $repeater->add_control(
            'legend_item_title',
            [
                'label' => esc_html__( 'Title', 'gmaper-elementor' ),
                'label_block' => 'true',
                'type' => Controls_Manager::TEXT,
                'dynamic' => [
                    'active' => true,
                ],
                'default' => esc_html__( 'Title', 'gmaper-elementor' ),
                'placeholder' => esc_html__( 'Type your title here', 'gmaper-elementor' ),
            ]
        );

        $this->add_control(
            'list_legend',
            [
                'label' => esc_html__( 'Legend items', 'gmaper-elementor' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'default' => [
                    [
                        'legend_item_title' => esc_html__( 'Title #1', 'gmaper-elementor' ),
                    ],
                ],
                'title_field' => '{{{ legend_item_title }}}',
            ]
        );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Section Marker Settings.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_marker_settings()
    {
        $this->start_controls_section( 'section_marker_settings', [
            'label' => esc_html__( 'Marker', 'gmaper-elementor' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $repeater = new Repeater();

        $repeater->add_control(
            'pin_icon',
            [
                'label' => esc_html__( 'Choose Image', 'gmaper-elementor' ),
                'type' => Controls_Manager::MEDIA,
            ]
        );

        $repeater->add_control(
            'rectangular_image',
            [
                'label' => esc_html__( 'Rectangular image', 'gmaper-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'gmaper-elementor' ),
                'label_off' => esc_html__( 'No', 'gmaper-elementor' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $repeater->add_control(
            'square_pin_size',
            [
                'label' => esc_html__( 'Custom marker size', 'gmaper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 50,
                ],
                'condition' => [
                        'rectangular_image!' => 'yes'
                ]
            ]
        );

        $repeater->add_control(
            'rectangular_pin_size_height',
            [
                'label' => esc_html__( 'Custom marker height', 'gmaper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 65,
                ],
                'condition' => [
                        'rectangular_image' => 'yes'
                ]
            ]
        );

        $repeater->add_control(
            'rectangular_pin_size_width',
            [
                'label' => esc_html__( 'Custom marker width', 'gmaper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 50,
                ],
                'condition' => [
                        'rectangular_image' => 'yes'
                ]
            ]
        );

        $repeater->add_control(
            'pin_latitude',
            [
                'label' => esc_html__( 'Latitude', 'gmaper-elementor' ),
                'label_block' => 'true',
                'default' => '-34.391',
                'dynamic' => [
                    'active' => true,
                ],
                'type' => Controls_Manager::TEXT,
            ]
        );

        $repeater->add_control(
            'pin_longitude',
            [
                'label' => esc_html__( 'Longitude', 'gmaper-elementor' ),
                'label_block' => 'true',
                'default' => '150.644',
                'dynamic' => [
                    'active' => true,
                ],
                'type' => Controls_Manager::TEXT,
            ]
        );

        $repeater->add_control(
            'pin_item_title',
            [
                'label' => esc_html__( 'Title', 'gmaper-elementor' ),
                'label_block' => 'true',
                'type' => Controls_Manager::TEXT,
                'dynamic' => [
                    'active' => true,
                ],
                'default' => 'Title',
                'placeholder' => esc_html__( 'Type your title here', 'gmaper-elementor' ),
            ]
        );

        $repeater->add_control(
            'pin_item_description',
            [
                'label' => esc_html__( 'Description', 'gmaper-elementor' ),
                'dynamic' => [
                    'active' => true,
                ],
                'type' => Controls_Manager::WYSIWYG,
                'placeholder' => esc_html__( 'Type description here', 'gmaper-elementor' ),
            ]
        );

        $this->add_control(
            'list_marker',
            [
                'label' => esc_html__( 'Map pins', 'gmaper-elementor' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'default' => [
                    [
                        'pin_item_title' => esc_html__( 'Title', 'gmaper-elementor' ),
                    ],
                ],
                'title_field' => '{{{ pin_item_title }}}',
            ]
        );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Content -> Section Map Styles Settings.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_map_styles_settings()
    {
        $this->start_controls_section('section_map_styles_settings', [
            'label' => esc_html__('Map styles', 'gmaper-elementor'),
            'tab' => Controls_Manager::TAB_CONTENT
        ]);

        $this->add_control(
            'map_style',
            [
                'label' => esc_html__( 'Map style', 'gmaper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'standard',
                'options' => [
                    'standard'  => esc_html__( 'Standard', 'gmaper-elementor' ),
                    'silver' => esc_html__( 'Silver', 'gmaper-elementor' ),
                    'retro' => esc_html__( 'Retro', 'gmaper-elementor' ),
                    'dark' => esc_html__( 'Dark', 'gmaper-elementor' ),
                    'night' => esc_html__( 'Night', 'gmaper-elementor' ),
                    'aubergine' => esc_html__( 'Aubergine', 'gmaper-elementor' ),
                    'custom' => esc_html__( 'Custom', 'gmaper-elementor' )
                ],
            ]
        );

        $this->add_control(
            'custom_map_style',
            [
                'label' => esc_html__( 'JSON code', 'gmaper-elementor' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'placeholder' => esc_html__( 'Type your JSON code here', 'gmaper-elementor' ),
                'condition' => [
                        'map_style' => 'custom'
                ]
            ]
        );

        $this->end_controls_section();

    }


    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @return void
     * @since 1.0.0
     * @access private
     *
     */
    private function generate_margin_padding_controls( $section_id, $html_class ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'gmaper-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'gmaper-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $html_class
     * @param bool $include_color
     * @param bool $include_bg
     * @param bool $include_css_filter
     * @param bool $include_typography
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $html_class, $include_color = true, $include_bg = true, $include_css_filter = false, $include_typography = true ) {

        if ( $include_typography ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'gmaper-elementor'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .$html_class",
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab( $section_id.'_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'gmaper-elementor' )] );

        if ( $include_color ) {

            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__('Color', 'gmaper-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$html_class" => 'color: {{VALUE}}',
                    ],
                ]
            );

        }

        if ( $include_bg ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'gmaper-elementor'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .$html_class",
                ]
            );

            $this->add_control(
                $section_id . '_separate_normal',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );

        }


        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'gmaper-elementor' ),
                'selector' => "{{WRAPPER}} .$html_class",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'gmaper-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'gmaper-elementor' ),
                'selector' => "{{WRAPPER}} .$html_class",
            ]
        );

        if ( $include_css_filter ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name' => 'css_filters_normal',
                    'selector' => '{{WRAPPER}} .'.$html_class,
                ]
            );
        }


        $this->end_controls_tab();

        $this->start_controls_tab( $section_id.'_hover_style_tab', ['label' => esc_html__( 'HOVER', 'gmaper-elementor' )] );

        if ( $include_color ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__('Color', 'gmaper-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$html_class:hover" => 'color: {{VALUE}}',
                    ],
                ]
            );
        }

        if ( $include_bg ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'gmaper-elementor'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .$html_class:hover",
                ]
            );
        }


        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'gmaper-elementor' ),
                'selector' => "{{WRAPPER}} .$html_class:hover",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'gmaper-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .$html_class:hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'gmaper-elementor' ),
                'selector' => "{{WRAPPER}} .$html_class:hover",
            ]
        );

        if ( $include_css_filter ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name' => 'css_filters_hover',
                    'selector' => '{{WRAPPER}} .'.$html_class.':hover',
                ]
            );
        }

        $this->end_controls_tab();

        $this->end_controls_tabs();
    }




    /**
     * Add widget controls: Style -> Section Style Map.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_map() {

        $this->start_controls_section( 'section_style_map', [
            'label' => esc_html__( 'Map', 'gmaper-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'section_style_map_margin',
            [
                'label' => esc_html__( 'Margin', 'gmaper-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .mdp-gmaper-elementor-google-map-wrapper" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    "{{WRAPPER}} .mdp-gmaper-elementor-google-map-overlay" => 'margin-right: {{RIGHT}}{{UNIT}}; margin-left: {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'map_height',
            [
                'label' => esc_html__( 'Height', 'gmaper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-gmaper-elementor-google-map-wrapper' => 'height: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-gmaper-elementor-google-map-overlay' => 'height: {{SIZE}}{{UNIT}};'
                ],
            ]
        );

        $this->add_control(
            'map_width',
            [
                'label' => esc_html__( 'Width', 'gmaper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-gmaper-elementor-google-map-wrapper' => 'width: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-gmaper-elementor-google-map-overlay' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->start_controls_tabs( 'section_styles_map_style_tabs' );

        $this->start_controls_tab( 'section_styles_map_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'gmaper-elementor' )] );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_styles_map_border_normal',
                'label' => esc_html__( 'Border Type', 'gmaper-elementor' ),
                'selector' => "{{WRAPPER}} .mdp-gmaper-elementor-google-map-wrapper",
            ]
        );

        $this->add_responsive_control(
            'section_styles_map_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'gmaper-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-gmaper-elementor-google-map-wrapper, {{WRAPPER}} .mdp-gmaper-elementor-google-map-overlay" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_styles_map_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'gmaper-elementor' ),
                'selector' => "{{WRAPPER}} .mdp-gmaper-elementor-google-map-wrapper",
            ]
        );

        $this->add_group_control(
              Group_Control_Css_Filter::get_type(),
                [
                   'name' => 'css_filters_normal',
                   'selector' => '{{WRAPPER}} .mdp-gmaper-elementor-google-map-wrapper',
                ]
            );

        $this->end_controls_tab();

        $this->start_controls_tab( 'section_styles_map_hover_style_tab', ['label' => esc_html__( 'HOVER', 'gmaper-elementor' )] );

        $this->add_control(
            'section_styles_map_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_styles_map_border_hover',
                'label' => esc_html__( 'Border Type', 'gmaper-elementor' ),
                'selector' => "{{WRAPPER}} .mdp-gmaper-elementor-google-map-wrapper:hover, {{WRAPPER}} .mdp-gmaper-elementor-google-map-overlay:hover + .mdp-gmaper-elementor-google-map-wrapper",
            ]
        );

        $this->add_responsive_control(
            'section_styles_map_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'gmaper-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-gmaper-elementor-google-map-wrapper:hover, {{WRAPPER}} .mdp-gmaper-elementor-google-map-overlay:hover + .mdp-gmaper-elementor-google-map-wrapper, {{WRAPPER}} .mdp-gmaper-elementor-google-map-overlay:hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_styles_map_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'gmaper-elementor' ),
                'selector' => "{{WRAPPER}} .mdp-gmaper-elementor-google-map-wrapper:hover, {{WRAPPER}} .mdp-gmaper-elementor-google-map-overlay:hover + .mdp-gmaper-elementor-google-map-wrapper",
            ]
        );

        $this->add_group_control(
            Group_Control_Css_Filter::get_type(),
            [
               'name' => 'css_filters_hover',
               'selector' => '{{WRAPPER}} .mdp-gmaper-elementor-google-map-wrapper:hover, {{WRAPPER}} .mdp-gmaper-elementor-google-map-overlay:hover + .mdp-gmaper-elementor-google-map-wrapper',
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Overlay.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_overlay()
    {

        $this->start_controls_section('section_style_overlay', [
            'label' => esc_html__('Overlay', 'gmaper-elementor'),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                    'map_overlay' => 'yes'
            ]
        ]);

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'background',
                'label' => esc_html__( 'Background', 'gmaper-elementor' ),
                'types' => [ 'classic', 'gradient' ],
                'selector' => '{{WRAPPER}} .mdp-gmaper-elementor-google-map-overlay',
            ]
        );



        $this->add_control(
            'overlay_blend_mode',
            [
                'label' => esc_html__( 'Blend Mode', 'gmaper-elementor' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    '' => esc_html__( 'Normal', 'gmaper-elementor' ),
                    'multiply' => 'Multiply',
                    'screen' => 'Screen',
                    'overlay' => 'Overlay',
                    'darken' => 'Darken',
                    'lighten' => 'Lighten',
                    'color-dodge' => 'Color Dodge',
                    'saturation' => 'Saturation',
                    'color' => 'Color',
                    'difference' => 'Difference',
                    'exclusion' => 'Exclusion',
                    'hue' => 'Hue',
                    'luminosity' => 'Luminosity',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-gmaper-elementor-google-map-overlay' => 'mix-blend-mode: {{VALUE}}',
                ],
                'separator' => 'none',
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Marker Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_marker_title()
    {
        $this->start_controls_section('section_style_marker_title', [
            'label' => esc_html__('Marker title', 'gmaper-elementor'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_marker_title', 'mdp-gmaper-elementor-title' );

        $this->add_control(
            'marker_title_text_align',
            [
                'label' => esc_html__( 'Alignment', 'gmaper-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'gmaper-elementor' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'gmaper-elementor' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'gmaper-elementor' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                    '{{WRAPPER}} .mdp-gmaper-elementor-title-wrapper' => 'text-align: {{VALUE}}'
                ]
            ]
        );


        $this->generate_typography_tabs_controls( 'section_style_marker_title', 'mdp-gmaper-elementor-title' );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Style Marker Description.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_marker_description()
    {
        $this->start_controls_section('section_style_marker_description', [
            'label' => esc_html__('Marker description', 'gmaper-elementor'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_marker_description', 'mdp-gmaper-elementor-description' );

        $this->add_control(
            'marker_description_text_align',
            [
                'label' => esc_html__( 'Alignment', 'gmaper-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'gmaper-elementor' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'gmaper-elementor' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'end' => [
                        'title' => esc_html__( 'Right', 'gmaper-elementor' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                        '{{WRAPPER}} .mdp-gmaper-elementor-description-wrapper' => 'text-align: {{VALUE}}'
                ]
            ]
        );


        $this->generate_typography_tabs_controls( 'section_style_marker_description', 'mdp-gmaper-elementor-description' );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Style Legend.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_legend()
    {
        $this->start_controls_section('section_style_legend', [
            'label' => esc_html__('Legend', 'gmaper-elementor'),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                'show_legend' => 'yes'
            ]
        ]);

        $this->generate_margin_padding_controls( 'section_style_legend', 'mdp-gmaper-elementor-google-map-legend' );


        $this->add_control(
            'legend_wrapper_height',
            [
                'label' => esc_html__( 'Height', 'gmaper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-gmaper-elementor-google-map-legend' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'legend_wrapper_width',
            [
                'label' => esc_html__( 'Width', 'gmaper-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-gmaper-elementor-google-map-legend' => 'max-width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );


        $this->generate_typography_tabs_controls( 'legend_icon_size', 'mdp-gmaper-elementor-google-map-legend', false, true, false, false );

        $this->end_controls_section();
    }


    /**
     * Add widget controls: Style -> Section Style Legend Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_legend_title()
    {
        $this->start_controls_section('section_style_legend_title', [
            'label' => esc_html__('Legend title', 'gmaper-elementor'),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                'show_legend' => 'yes'
            ]
        ]);

        $this->generate_margin_padding_controls( 'section_style_legend_title', 'mdp-gmaper-elementor-google-map-legend--title' );

        $this->add_control(
            'legend_title_text_align',
            [
                'label' => esc_html__( 'Alignment', 'gmaper-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'gmaper-elementor' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'gmaper-elementor' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'end' => [
                        'title' => esc_html__( 'Right', 'gmaper-elementor' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                    '{{WRAPPER}} .mdp-gmaper-elementor-google-map-legend--item-title-wrapper' => 'text-align: {{VALUE}}'
                ]
            ]
        );


        $this->generate_typography_tabs_controls( 'section_style_legend_title', 'mdp-gmaper-elementor-google-map-legend--title' );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Style Legend Items.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_legend_items()
    {
        $this->start_controls_section('section_style_legend_items', [
            'label' => esc_html__('Legend items', 'gmaper-elementor'),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                'show_legend' => 'yes'
            ]
        ]);

        $this->generate_margin_padding_controls( 'section_style_legend_items', 'mdp-gmaper-elementor-google-map-legend--item' );

        $this->generate_typography_tabs_controls( 'section_style_legend_items', 'mdp-gmaper-elementor-google-map-legend--item' );

        $this->end_controls_section();
    }


    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();
        $markers = json_encode( $settings['list_marker']);

        ?>
        <!-- Start Gmaper for Elementor WordPress Plugin -->
        <div class="mdp-gmaper-elementor-box"
             data-latitude="<?php echo esc_attr( $settings['center_latitude'] ); ?>"
             data-longitude="<?php echo esc_attr( $settings['center_longitude'] ); ?>"
             data-map-zoom = "<?php echo esc_attr( $settings['map_zoom']['size'] ); ?>"
             data-map-type = "<?php echo esc_attr( $settings['map_type'] ); ?>"
             data-fullscreen-control = "<?php echo esc_attr( $settings['fullscreen_control'] ); ?>"
             data-zoom-control = "<?php echo esc_attr( $settings['zoom_controls'] ); ?>"
             data-streetview-control = "<?php echo esc_attr( $settings['street_view_control'] ); ?>"
             data-map-type-control = "<?php echo esc_attr( $settings['map_type_control'] ); ?>"
             data-scroll-wheel-zoom = "<?php echo esc_attr( $settings['scroll_wheel_zoom'] ); ?>"
             data-map-dragged = "<?php echo esc_attr( $settings['disable_map_drag'] ); ?>"
             data-markers = "<?php echo esc_attr( $markers ); ?>"
             data-info-container-always-opened = "<?php echo esc_attr( $settings['info_always_open'] ); ?>"
             data-info-container-open-hover = "<?php echo esc_attr( $settings['info_open_hover'] ); ?>"
             data-user-geolocation = "<?php echo esc_attr( $settings['center_user_location'] ); ?>"
             data-show-legend = "<?php echo esc_attr( $settings['show_legend'] ); ?>"
             data-custom-styles = "<?php echo esc_attr( $settings['custom_map_style'] ); ?>"
             data-map-style = "<?php echo esc_attr( $settings['map_style'] ); ?>"
             data-close-info-container = "<?php echo esc_attr( $settings['info_container_close'] ) ?>"
        >
            <?php if ( $settings['map_overlay'] === 'yes' ): ?>
                <div class="mdp-gmaper-elementor-google-map-overlay"></div>
            <?php endif; ?>
            <div class="mdp-gmaper-elementor-google-map-wrapper
            <?php if ( $settings['hide_logo'] === 'yes' ) echo esc_attr( 'mdp-gmaper-elementor-disable-logo' ); ?>
            <?php if ( $settings['hide_copyright'] === 'yes' ) echo esc_attr( 'mdp-gmaper-elementor-disable-copyright' ); ?>
            <?php if ( $settings['hide_terms_of_use'] === 'yes' ) echo esc_attr( 'mdp-gmaper-elementor-disable-terms-of-use' ); ?>
            <?php if ( $settings['hide_report_error'] === 'yes' ) echo esc_attr( 'mdp-gmaper-elementor-disable-report-error' ); ?>
            <?php if ( $settings['hide_keyboard_shortcuts'] === 'yes' ) echo esc_attr( 'mdp-gmaper-elementor-disable-keyboard-shortcuts' ); ?>
            "></div>
            <?php if ( $settings['show_legend'] === 'yes' ): ?>
            <div class="mdp-gmaper-elementor-google-map-legend">
                <div class="mdp-gmaper-elementor-google-map-legend--item-title-wrapper">
                    <div class="mdp-gmaper-elementor-google-map-legend--title"><?php esc_html_e( $settings['legend_title'] ); ?></div>
                </div>
                <div class="mdp-gmaper-elementor-google-map-legend--items">
                    <?php foreach( $settings['list_legend'] as $item ): ?>
                        <div class="mdp-gmaper-elementor-google-map-legend--item">
                            <div class="mdp-gmaper-elementor-google-map-legend--item-image" style="background-size: cover; background: url('<?php echo esc_url( $item['legend_icon']['url'] ) ?> ') center;"></div>
                            <div class="mdp-gmaper-elementor-google-map-legend--item-title"><?php esc_html_e( $item['legend_item_title'] ); ?></div></div>
                    <?php endforeach; ?>
                </div>
            </div>
            <?php endif; ?>
        </div>
        <!-- End Gmaper for Elementor WordPress Plugin -->
        <?php
        if ( is_admin() ) {
            $widget_hash = substr( hash( 'ripemd160', date('l jS \of F Y h:i:s A') ), rand( 0 , 20 ), 3 ) . rand( 11 , 99 );
            ?>

            <!--suppress JSUnresolvedFunction -->
            <script>
                try {
                    gmaperReady<?php esc_attr_e( $widget_hash ); ?>( mdpGmaper.addGmaper.bind( mdpGmaper) );
                } catch ( msg ) {
                    const gmaperReady<?php esc_attr_e( $widget_hash ); ?> = ( callback ) => {
                        'loading' !== document.readyState ?
                            callback() :
                            document.addEventListener( 'DOMContentLoaded', callback );
                    };
                    gmaperReady<?php esc_attr_e( $widget_hash ); ?>( mdpGmaper.addGmaper.bind( mdpGmaper ) );
                }
            </script>
            <?php
        }
    }

    /**
     * JS Render.
     *
     * @access protected
     *
     * @return void
     **/
    protected function content_template() {
        ?>
        <div class="mdp-gmaper-elementor-box"
             data-latitude="{{{ settings.center_latitude }}}"
             data-longitude="{{{ settings.center_longitude }}}"
             data-map-zoom = "{{{ settings.map_zoom.size }}}"
             data-map-type = "{{{ settings.map_type }}}"
             data-fullscreen-control = "{{{ settings.fullscreen_control }}}"
             data-zoom-control = "{{{ settings.zoom_controls }}}"
             data-streetview-control = "{{{ settings.street_view_control }}}"
             data-map-type-control = "{{{ settings.map_type_control }}}"
             data-scroll-wheel-zoom = "{{{ settings.scroll_wheel_zoom }}}"
             data-map-dragged = "{{{ settings.disable_map_drag }}}"
             data-markers = "{{ JSON.stringify( settings.list_marker ) }}"
             data-info-container-always-opened = "{{{ settings.info_always_open }}}"
             data-info-container-open-hover = "{{{ settings.info_open_hover }}}"
             data-user-geolocation = "{{{ settings.center_user_location }}}"
             data-show-legend = "{{{ settings.show_legend }}}"
             data-custom-styles = "{{{ settings.custom_map_style }}}"
             data-map-style = "{{{ settings.map_style }}}"
             data-close-info-container = "{{{ settings.info_container_close }}}"
        >
            <# if ( settings.map_overlay === 'yes' ) { #>
            <div class="mdp-gmaper-elementor-google-map-overlay"></div>
            <# } #>
            <div class="mdp-gmaper-elementor-google-map-wrapper
            <# if (settings.hide_logo === 'yes' ) { #>
            mdp-gmaper-elementor-disable-logo
            <# } #>
            <# if (settings.hide_copyright === 'yes' ) { #>
            mdp-gmaper-elementor-disable-copyright
            <# } #>
            <# if (settings.hide_terms_of_use === 'yes' ) { #>
            mdp-gmaper-elementor-disable-terms-of-use
            <# } #>
            <# if (settings.hide_report_error === 'yes' ) { #>
            mdp-gmaper-elementor-disable-report-error
            <# } #>
            <# if (settings.hide_keyboard_shortcuts === 'yes' ) { #>
            mdp-gmaper-elementor-disable-keyboard-shortcuts
            <# } #>
            "></div>

            <# if ( settings.show_legend === 'yes' ) { #>
                <div class="mdp-gmaper-elementor-google-map-legend">
                    <div class="mdp-gmaper-elementor-google-map-legend--item-title-wrapper">
                        <div class="mdp-gmaper-elementor-google-map-legend--title">{{{ settings.legend_title }}}</div>
                    </div>
                    <div class="mdp-gmaper-elementor-google-map-legend--items">
                        <# _.each( settings.list_legend, function( item, index ) { #>
                            <div class="mdp-gmaper-elementor-google-map-legend--item">
                                <div class="mdp-gmaper-elementor-google-map-legend--item-image" style="background: url({{{ item.legend_icon.url }}}) center; background-size: cover;"></div>
                                <div class="mdp-gmaper-elementor-google-map-legend--item-title">{{{ item.legend_item_title }}}</div>
                            </div>
                        <# } ) #>
                    </div>
                </div>
            <# } #>
        </div>

    <?php
        if ( is_admin() ) {
            $widget_hash = substr( hash( 'ripemd160', date('l jS \of F Y h:i:s A') ), rand( 0 , 20 ), 3 ) . rand( 11 , 99 );
            ?>
            <!--suppress JSUnresolvedFunction -->
            <script>
                try {
                    gmaperReady<?php esc_attr_e( $widget_hash ); ?>( mdpGmaper.addGmaper.bind( mdpGmaper ) );
                } catch ( msg ) {
                    const gmaperReady<?php esc_attr_e( $widget_hash ); ?> = ( callback ) => {
                        'loading' !== document.readyState ?
                            callback() :
                            document.addEventListener( 'DOMContentLoaded', callback );
                    };
                    gmaperReady<?php esc_attr_e( $widget_hash ); ?>( mdpGmaper.addGmaper.bind( mdpGmaper ) );
                }
            </script>
            <?php
        }
    }


    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/gmaper';

    }

}
